class Database {
  constructor(dbPath) {
    this.dbPath = dbPath;
    this.collections = {};
    this.locks = {}; // Track locks for each collection
    this.operationQueues = {}; // Track operation queues for each record
  }

  // ...

  async executeWithLock(collectionName, recordId, callback) {
    const recordKey = `${collectionName}:${recordId}`;

    if (!this.locks[recordKey]) {
      // Acquire the lock if it's not already held
      this.locks[recordKey] = true;

      try {
        const result = await callback();

        // Release the lock
        this.locks[recordKey] = false;

        // Process pending operations in the queue
        const queue = this.operationQueues[recordKey];
        if (queue && queue.length > 0) {
          for (const operation of queue) {
            await operation();
          }
          delete this.operationQueues[recordKey]; // Clear the queue
        }

        return result;
      } catch (error) {
        // Handle any errors
        this.locks[recordKey] = false;
        throw error;
      }
    } else {
      // Queue the operation if the lock is held
      return new Promise((resolve, reject) => {
        const queue = this.operationQueues[recordKey] || [];
        queue.push(async () => {
          try {
            const result = await callback();
            resolve(result);
          } catch (error) {
            reject(error);
          }
        });
        this.operationQueues[recordKey] = queue;
      });
    }
  }

  // ...
}
